#ifndef _SNET_H
#define _SNET_H

#include <netinet/in.h>
#include <netlink/msg.h>
#include <netlink/genl/genl.h>
#include <netlink/genl/ctrl.h>
#include <netlink/netlink.h>
#include <netlink/utils.h>
#include <asm/types.h>

#define NIPQUAD(addr) \
	((unsigned char *)&addr)[0], \
	((unsigned char *)&addr)[1], \
	((unsigned char *)&addr)[2], \
	((unsigned char *)&addr)[3]

#define NIP6(addr) \
	ntohs((addr).s6_addr16[0]), \
	ntohs((addr).s6_addr16[1]), \
	ntohs((addr).s6_addr16[2]), \
	ntohs((addr).s6_addr16[3]), \
	ntohs((addr).s6_addr16[4]), \
	ntohs((addr).s6_addr16[5]), \
	ntohs((addr).s6_addr16[6]), \
	ntohs((addr).s6_addr16[7])
#define NIP6_FMT "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x"

enum snet_syscall {
	SNET_SOCKET_CREATE = 0,
	SNET_SOCKET_BIND,
	SNET_SOCKET_CONNECT,
	SNET_SOCKET_LISTEN,
	SNET_SOCKET_ACCEPT,
	SNET_SOCKET_SENDMSG,
	SNET_SOCKET_RECVMSG,
	SNET_SOCKET_SOCK_RCV_SKB,
	SNET_SOCKET_CLOSE,
};

enum snet_verdict {
	SNET_VERDICT_GRANT = 0,	/* grant the syscall */
	SNET_VERDICT_DENY,	/* deny the syscall */
	SNET_VERDICT_PENDING,	/* waiting for a decision */
	SNET_VERDICT_NONE,	/* no decision can be set */
};

#define SNET_VERSION 0x1	/* -> include <linux/snet.h> */

/* commands */
enum {
	SNET_C_UNSPEC,
	SNET_C_VERSION,
	SNET_C_REGISTER,
	SNET_C_UNREGISTER,
	SNET_C_INSERT,
	SNET_C_REMOVE,
	SNET_C_FLUSH,
	SNET_C_LIST,
	SNET_C_VERDICT,
	SNET_C_VERDICT_DELAY,
	__SNET_C_MAX,
};

#define SNET_C_MAX (__SNET_C_MAX - 1)

/* attributes */
enum {
	SNET_A_UNSPEC,
	SNET_A_VERSION,         /* (NLA_U32) the snet protocol version  */
	SNET_A_SYSCALL,         /* (NLA_U8)  a syscall identifier       */
	SNET_A_PROTOCOL,        /* (NLA_U8)  a protocol identifier      */
	SNET_A_INSERTED,
	SNET_A_REMOVED,
	SNET_A_FLUSHED,
	SNET_A_REGISTERED,
	SNET_A_UNREGISTERED,
	SNET_A_VERDICT_ID,
	SNET_A_FAMILY,
	SNET_A_UID,
	SNET_A_PID,
	SNET_A_VERDICT,
	SNET_A_DATA_EXT,
	SNET_A_VERDICT_DELAY,
	SNET_A_VERDICT_DELAYED,
	__SNET_A_MAX,
};

#define SNET_A_MAX (__SNET_A_MAX - 1)

#define SNET_GENL_NAME          "SNET"
#define SNET_GENL_VERSION       SNET_VERSION

struct snet_sock_half {
	struct {
		union {
			__be32 ip;
			struct in6_addr ip6;
		};
	} u3;
	struct {
		__be16 port;
	} u;
};

struct snet_sock_info {
	struct snet_sock_half src;
	struct snet_sock_half dst;
	int type;
};

struct nl_handle *snet_create_nlhandle(unsigned int (*verdict_cb)(uint32_t, uint8_t, uint8_t,
								  uint8_t, uint32_t, uint32_t,
								  struct snet_sock_info *));
int snet_resolve(struct nl_handle *nl_handle);
int snet_register(struct nl_handle *nl_handle, int f, int version);
int snet_unregister(struct nl_handle *nl_handle, int f, int version);
int snet_get_version(struct nl_handle *nl_handle, int f, int version);
int snet_insert(struct nl_handle *nl_handle, int family,
		__u8 syscall, __u8 protocol, int version);
int snet_remove(struct nl_handle *nl_handle, int family,
		__u8 syscall, __u8 protocol, int version);
int snet_verdict(struct nl_handle *nl_handle, int family, int version,
		 uint32_t verdict_id, uint8_t verdict);

#endif /* _SNET_H */
